PRO RCPLT_OPERATIONAL
;+
; NAME:
;	rcplt_operational
;
; PURPOSE:
;	This program reads the file rcplt.dat and plots 
;	the storm-total rainfall as a swath using IDL graphics.
;
;	MODIFICATION HISTORY:
;	Written, Frank Marks, Feb. 9, 2002.
;	Modified, Frank Marks, AOML, 11/02/02
;-
SET_PLOT, 'MAC'
DEVICE, RETAIN=2, DECOMPOSED=0

;
;     Open the input file
;     Define variables (fname) as strings before reading:
fname = ''
basin = ''
sname = ''
isnum = 0
iyr = 0
imon = 0
iday = 0
ihr = 0
mxp = 0
nyp = 0
  
;     Read fname from the terminal and open it
READ, fname, PROMPT='Enter input filename: '

OPENR, lun, fname, ERROR=err, /get_lun

;     Read the number of track points (numt) from the terminal
;READ, numt, PROMPT='Enter # of track points: '
;     Read the input file headers from 1st and 2nd line of file

readf, lun, format = '(a2,i2,i4,1x,3(i2),1x,a10)', basin,isnum,iyr,imon,iday,ihr,sname
readf, lun, format = '(1x,i3,1x,3(f8.3,1x),i3,1x,3(f8.3,1x))', mxp,sx,wlon,elon,nyp,sy,slat,nlat

;     read the storm track starting at 3rd line of file and the next tracknum lines
;     default number of operational track points (72 h forecast)
numt = 6
stlon=make_array(numt,/float)
stlat=make_array(numt,/float)
stime=make_array(numt,/float)

for j=0,numt-1 do begin
  readf, lun, format = '(1x,f7.2,1x,f6.2,1x,f5.1)', tlon, tlat, timej
  stlon(j)=tlon
  stlat(j)=tlat
  stime(j)=timej
endfor

;     skip next line as it has a zero storm location
readf, lun, format = '(f8.2,f7.2,f6.1)', tlon, tlat, timej

;     read the rain array starting at end of track to the end of file
;     create 2-D array for plotting
raintot=MAKE_ARRAY(mxp, nyp, /FLOAT, VALUE = 0)

;     generate lat, lon arrays using the scales and west and southmost points
lon=(findgen(mxp)*sx)+wlon
lat=(findgen(nyp)*sy)+slat
r=long(mxp*nyp)

;     fill 2-D rain array
num=0
n=LONG(num)
FOR j=0,nyp-1 DO BEGIN
  FOR i=0,mxp-1 DO BEGIN
	n=n+1
	IF (n LT r) THEN BEGIN
	   readf, lun, format = '(3(1x,f8.3))', dlat, dlon, rain
	   raintot(i,j) = rain
	ENDIF
  ENDFOR
ENDFOR

; free the input unit (lun)
free_lun, lun

;
; Make the plot
;
; open window
; Set plotting to PostScript:
SET_PLOT, 'PS'
  
; Use DEVICE to set some PostScript device options:
DEVICE, FILENAME=string(iyr,imon,iday,ihr,basin,isnum,'.ps',format='(i4,i2.2,2i2.2,a2,i2.2,a3)'), $
        /landscape, /color

;window, 0, xsize=600, ysize=480, title=string(sname,' R-CLIPER')  ; for screen output only
;
; Load discrete colors for contours (39 Rainbow+White color table for monitor, 40 for PS)
loadct, 40   ;39 for sceen and 40 for .ps
;
; set map projection and limits set by input file
map_set, /cylindrical, limit=[slat,wlon,nlat,elon], /isotropic
;
; plot continents in the 30x30 window, add lat, lon grid
map_continents, color=76, /hires, fill_continents=1
;
; overlay contours (clevel) of rain, first filled, then overlay lines
clevel=[0.1,0.25,0.5,1.,2.,3.,4.,5.,6.,7.,8.,10.,12.,15.,20.]
CONTOUR, raintot, lon, lat, LEVELS=clevel, C_COLORS=indgen(16)*15+32, /FILL, /OVERPLOT
CONTOUR, raintot, lon, lat, LEVELS=clevel, FONT=0, C_CHARSIZE=0.75, color=0, /OVERPLOT
;
; plot track over the contours
oplot, stlon, stlat, linestyle=1, thick=2.5, color=240, psym=7
oplot, stlon, stlat, linestyle=1, thick=2.5, color=240

;overlay state boundaries and continental boundaries in black
map_continents, color=0, /usa, /hires
map_continents, color=0, /hires, /coasts
map_grid, /box_axes, color=255, /label, latdel=5., londel=5.

;label plot (x,y)=(80,60) and (80,40) for screen; =(
xyouts, wlon+1., slat+3., string(iyr,imon,iday,format='(i4,i2.2,i2.2)'), charsize=2, charthick=1.5,$
        color=255
xyouts, wlon+1., slat+1.5, string(ihr,'00 UTC',sname,format='(i2.2,a6,1x,a10)'), charsize=2,$
        charthick=1.5, color=255
xyouts, wlon+1.1,slat+4.25, string('X - 12 h storm positions'), color=240, charthick=1.5

; Close the PostScript file:
DEVICE,/CLOSE_FILE
  
; Return plotting to the original device:
SET_PLOT, 'MAC'

END